@@ -1,14 +1,22 @@
 //! This module contains conversions between various Rust object and their representation in Python.
 
 pub mod anyhow;
+pub mod bigdecimal;
+pub mod bytes;
 pub mod chrono;
+pub mod chrono_tz;
 pub mod either;
 pub mod eyre;
 pub mod hashbrown;
 pub mod indexmap;
+pub mod jiff;
 pub mod num_bigint;
 pub mod num_complex;
+pub mod num_rational;
+pub mod ordered_float;
 pub mod rust_decimal;
 pub mod serde;
 pub mod smallvec;
 mod std;
+pub mod time;
+pub mod uuid;
