@@ -6,14 +6,20 @@
 //! APIs may may change at any time without documentation in the CHANGELOG and without
 //! breaking semver guarantees.
 
-pub mod deprecations;
+pub mod callback;
+pub mod concat;
+#[cfg(feature = "experimental-async")]
+pub mod coroutine;
+pub mod exceptions;
 pub mod extract_argument;
 pub mod freelist;
 pub mod frompyobject;
-pub(crate) mod not_send;
+#[cfg(feature = "experimental-inspect")]
+pub mod introspection;
 pub mod panic;
 pub mod pycell;
 pub mod pyclass;
+pub mod pyclass_init;
 pub mod pyfunction;
 pub mod pymethods;
 pub mod pymodule;
